==> bless-0.3.1.tar.gz: bless-0.3.1/src/util/ByteArray.cs <==
// created on 1/13/2005 at 10:59 AM
/*
 *   Copyright (c) 2005, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
using System;
using System.Text;
using NUnit.Framework;
 
namespace Bless.Util {

///<summary>Create byte arrays from various kinds of strings</summary>
public class ByteArray
{
	// Don't allow instantiation
	private ByteArray() { }
 
	static private int HexCharToInt(char c)
	{	
		if (c>='0' && c <='9')
			return c-'0';
		
		c=Char.ToLower(c);
		
		if (c>='a' && c<='f')
			return c-'a'+10;
	
		throw new FormatException(String.Format("Character '{0}' is not valid in a hexadecimal string.", c)) ;
	}
		
	static public byte[] FromHexString(string s)
	{
		int i=0;
		int len=s.Length;
		byte[] ba=new byte[len/2];
		int j=0;
		
		// ignore leading whitespace
		while (i<len && s[i]==' ') i++;
		
		// if a single character is given
		if (i==len-1)
			throw new FormatException("Hexadecimal strings consist of at least two digits.");
		
		// while there are at least two characters left
		while (i+1 < len) {
			int h1=HexCharToInt(s[i++]);
			if (s[i]==' ')
				throw new FormatException(String.Format("Missing hexadecimal digit after '{0}'.", s[i-1]));
			int h2=HexCharToInt(s[i++]);
			
			ba[j++]=(byte)(h1*16+h2);	
			
			// skip spaces		
			while (i<len && s[i]==' ') i++;
		}
				
		if (i==len-1) {
			int k=HexCharToInt(s[i++]);
			throw new FormatException(String.Format("Missing hexadecimal digit after '{0}'.", s[i-1]));
		}
		
		byte[] baRet=new byte[j];
		
		Array.Copy(ba, baRet, j);
		
		return baRet;
	}
	
	static private char NibbleToChar(byte b, int i)
	{
		byte b1=(byte)((b >> (4*i)) & 0x0f);
		if (b1>9)
			return (char)('a'+b1-10);
		else
			return (char)('0'+b1);
	}
	
	static public string ToHexString(byte[] ba)
	{
		if (ba.Length==0)
			return string.Empty;
		
		StringBuilder sb=new StringBuilder(ba.Length*3);
		int i=0;
		
		foreach(byte b in ba) {
			sb.Append(NibbleToChar(b, 1));
			sb.Append(NibbleToChar(b, 0));
			sb.Append(' ');
		}
		
		return sb.ToString(0, sb.Length-1);
	}
	
} //end ByteArray

[TestFixture]
public class ByteArrayTests
{
	[Test]
	public void ByteArrayHexTest1()
	{
		byte[] ba=ByteArray.FromHexString("0305ffFa66");
		
		Assert.AreEqual(5, ba.Length,"#Length");
		Assert.AreEqual(0x03, ba[0],"#0");
		Assert.AreEqual(0x05, ba[1],"#1");
		Assert.AreEqual(0xff, ba[2],"#2");
		Assert.AreEqual(0xfa, ba[3],"#3");
		Assert.AreEqual(0x66, ba[4],"#4");
			
	}
	
	[Test]
	public void ByteArrayHexTest2()
	{
		byte[] ba=ByteArray.FromHexString(" 63a5 fe DD Cb  ");
		
		Assert.AreEqual(5, ba.Length,"#Length");
		Assert.AreEqual(0x63, ba[0],"#0");
		Assert.AreEqual(0xa5, ba[1],"#1");
		Assert.AreEqual(0xfe, ba[2],"#2");
		Assert.AreEqual(0xdd, ba[3],"#3");
		Assert.AreEqual(0xcb, ba[4],"#4");
			
	}
	
	[Test]
	[ExpectedException(typeof(FormatException))]
	public void ByteArrayHexTest3()
	{
		byte[] ba=ByteArray.FromHexString(" 63a fe DD Cb  ");
			
	}
	
	[Test]
	[ExpectedException(typeof(FormatException))]
	public void ByteArrayHexTest4()
	{
		byte[] ba=ByteArray.FromHexString(" 63 a5 fg DD Cb  ");
			
	}
	
	[Test]
	public void ByteArrayHexTest5()
	{
		string s=ByteArray.ToHexString(new byte[]{0x63, 0xff,0x12,0x00,0xca});
		
		Assert.AreEqual(14, s.Length,"#Length");
		Assert.AreEqual(s, "63 ff 12 00 ca","#0");	
	}
	
}


} // end namespace

==> bless-0.3.4.tar.gz: bless-0.3.4/src/util/ByteArray.cs <==
// created on 1/13/2005 at 10:59 AM
/*
 *   Copyright (c) 2005, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
using System;
using System.Text;
 
namespace Bless.Util {

///<summary>Create byte arrays from various kinds of strings</summary>
public class ByteArray
{
	// Don't allow instantiation
	private ByteArray() { }
		
	static public byte[] FromString(string s, int baseNum)
	{
		int i=0;
		int len=s.Length;
		byte[] ba=new byte[(len/BaseConverter.MinDigits[baseNum])+1];
		int j=0;
		
		// ignore leading whitespace
		while (i<len && s[i]==' ') i++;
		
		while (i < len) {
			int k=1;
			while (i+k < len &&  s[i+k]!=' ' && k < BaseConverter.MinDigits[baseNum]) k++;
			
			ba[j++]=(byte)BaseConverter.ConvertToNum(s, i, i+k-1, baseNum);
			
			// skip spaces
			i=i+k;		
			while (i<len && s[i]==' ') i++;
		}
		
		byte[] baRet=new byte[j];
		
		Array.Copy(ba, baRet, j);
		
		return baRet;
	}
	
	static public string ToString(byte[] ba, int baseNum)
	{
		if (ba.Length==0)
			return string.Empty;
		
		StringBuilder sb=new StringBuilder(ba.Length*(BaseConverter.MinDigits[baseNum]+1));
		
		foreach(byte b in ba) {
			sb.Append(BaseConverter.ConvertToString(b, baseNum));
			sb.Append(' ');
		}
		
		return sb.ToString(0, sb.Length-1);
	}
	
} //end ByteArray

} // end namespace
